# -*- coding: utf-8 -*-
##
## This file is part of the SixPay Indico EPayment Plugin.
## Copyright (C) 2017 Max Fischer
##
## This is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 3 of the
## License, or (at your option) any later version.
##
## This software is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Indico;if not, see <http://www.gnu.org/licenses/>.

from MaKaC.webinterface.pages import conferences
from MaKaC.webinterface.pages import registrationForm
from MaKaC.webinterface import wcomponents
from xml.sax.saxutils import quoteattr

from ..wcomponents import WTemplated
from .. import urlHandlers as localUrlHandlers
from ... import MODULE_ID


# Editor's Note  - MF@20170309
# * The W<.....> classes build templates pages
# Thanks to Magic [TM] indico pulls the templates from the `tpls` folder
# This is done using the class name MINUS THE FIRST CHARACTER. Example:
# class WConfModifEPaymentSixPay => ConfModifEPaymentSixPay.tpl
# * These classes are used by the RequestHandlers if they need to display stuff


class WPConfModifEPaymentSixPayBase(registrationForm.WPConfModifRegFormBase):
    def _createTabCtrl(self):
        self._tabCtrl = wcomponents.TabControl()
        self._tabMain = self._tabCtrl.newTab(
            "main",
            "Main",
            localUrlHandlers.UHConfModifEPaymentSixPay.getURL(self._conf)
        )
        wf = self._rh.getWebFactory()
        if wf:
            wf.customiseTabCtrl(self._tabCtrl)
        self._setActiveTab()

    def _setActiveTab(self):
        pass

    def _setActiveSideMenuItem(self):
        self._regFormMenuItem.setActive(True)

    def _getPageContent(self, params):
        self._createTabCtrl()
        banner = wcomponents.WEpaymentBannerModif(self._conf.getModPay().getPayModByTag(MODULE_ID),
                                                  self._conf).getHTML()
        html = wcomponents.WTabControl(self._tabCtrl, self._getAW()).getHTML(self._getTabContent(params))
        return banner + html

    def _getTabContent(self, params):
        return "nothing"


class WPConfModifEPaymentSixPay(WPConfModifEPaymentSixPayBase):
    def _getTabContent(self, params):
        wc = WConfModifEPaymentSixPay(self._conf)
        p = {
            'dataModificationURL': quoteattr(
                str(localUrlHandlers.UHConfModifEPaymentSixPayDataModif.getURL(self._conf)))
        }
        return wc.getHTML(p)

    def display(self, **params):
        result = WPConfModifEPaymentSixPayBase.display(self, **params)
        return result


class WConfModifEPaymentSixPay(WTemplated):
    def __init__(self, conference):
        WTemplated.__init__(self)
        self._conf = conference

    def getVars(self):
        """
        Current settings of this EPayment Module for display

        Displayed via `tpls/ConfModifEPaymentSixPay.tpl`.
        """
        _vars = WTemplated.getVars(self)
        modSixPay = self._conf.getModPay().getPayModByTag(MODULE_ID)
        _vars.update(modSixPay.getValues())
        return _vars


class WPConfModifEPaymentSixPayDataModif(WPConfModifEPaymentSixPayBase):
    def _getTabContent(self, params):
        wc = WConfModifEPaymentSixPayDataModif(self._conf)
        p = {'postURL': quoteattr(str(localUrlHandlers.UHConfModifEPaymentSixPayPerformDataModif.getURL(self._conf)))}
        return wc.getHTML(p)


class WConfModifEPaymentSixPayDataModif(WConfModifEPaymentSixPay):
    def __init__(self, conference):
        WTemplated.__init__(self)
        self._conf = conference

    def getVars(self):
        """
        Current settings of this EPayment Module for modification

        Displayed via `tpls/ConfModifEPaymentSixPayDataModif.tpl`.
        """
        return WConfModifEPaymentSixPay.getVars(self)


# Pages for the Six Pay service to call back after/during a transaction
class WPTransactionUserCallback(conferences.WPConferenceDefaultDisplayBase):
    """Request Handler for Callbacks the User is directed to"""
    #: display page after handling transaction
    display_template = None
    #: overwrite message on the displayed page
    message = None
    #: overwrite message detail on the displayed page
    message_detail = None

    def __init__(self, rh, conf, reg):
        conferences.WPConferenceDefaultDisplayBase.__init__(self, rh, conf)
        self._registrant = reg

    def _getBody(self, params):
        assert self.display_template is not None, "Callbacks must set display template for users"
        wc = self.display_template(self._conf, self._registrant)
        if self.message is not None:
            wc.message = self.message
        if self.message_detail is not None:
            wc.message_detail = self.message_detail
        return wc.getHTML()

    def _defineSectionMenu(self):
        conferences.WPConferenceDefaultDisplayBase._defineSectionMenu(self)
        self._sectionMenu.setCurrentItem(self._regFormOpt)


class WTransactionUserCallback(WTemplated):
    message = None
    message_detail = None

    def __init__(self, configuration, registrant):
        WTemplated.__init__(self)
        self._registrant = registrant
        self._conf = configuration

    def getVars(self):
        vars = WTemplated.getVars(self)
        assert self.message is not None, "Callbacks must set display message for users"
        vars["message"] = self.message
        vars["message_detail"] = self.message_detail or (
            "Registrant: %s %s" % (self._registrant.getFirstName(), self._registrant.getSurName())
        )
        return vars


class WTransactionSuccesslink(WTransactionUserCallback):
    """redirect when the user completed the transaction"""
    message = "Your payment has been successfully processed"


class WPTransactionSuccesslink(WPTransactionUserCallback):
    """redirect when the user completed the transaction"""
    display_template = WTransactionSuccesslink


class WTransactionFaillink(WTransactionUserCallback):
    """redirect when the user could not be authorised"""
    message = "You could not be authorised by the payment service"


class WPTransactionFaillink(WPTransactionUserCallback):
    """redirect when the user could not be authorised"""
    display_template = WTransactionFaillink


class WTransactionBacklink(WTransactionUserCallback):
    """redirect when the user aborts the transaction"""
    message = "You have aborted the transaction"


class WPTransactionBacklink(WPTransactionUserCallback):
    """redirect when the user aborts the transaction"""
    display_template = WTransactionBacklink



# -*- coding: utf-8 -*-
##
## This file is part of the SixPay Indico EPayment Plugin.
## Copyright (C) 2017 Max Fischer
##
## This is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 3 of the
## License, or (at your option) any later version.
##
## This software is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Indico;if not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import, division
from MaKaC.webinterface import wcomponents
import os
import pkg_resources
import MaKaC.common.Configuration as Configuration

import indico_sixpay as pay_plugin


# Editor's Note  - MF@20170309
# This file exists only as an auxiliary to .pages.ePayments
# I have no idea why it is separate


class WTemplated(wcomponents.WTemplated):
    # Editor's Note  - MF@20170309
    # If you are wondering how tplId is set:
    #   WTemplated sets it at an arbitrary time after __init__ in getHTML (!!!) to
    # self.tplId = self.__class__.__name__[1:]
    def _setTPLFile(self):
        """
        Guesses the TPL (template) file for the object. It will try to get
        from the configuration if there's a special TPL file for it and
        if not it will look for a file called as the class name+".tpl"
        in the configured TPL directory.
        """
        cfg = Configuration.Config.getInstance()
        tpl_dir = pkg_resources.resource_filename(pay_plugin.__name__, "tpls")
        tpl_file = cfg.getTPLFile(self.tplId)
        if tpl_file == "":
            tpl_file = "%s.tpl" % self.tplId
        self.tplFile = os.path.join(tpl_dir, tpl_file)
        hfile = self._getSpecificTPL(
            os.path.join(tpl_dir, 'chelp'),
            self.tplId,
            extension='wohl',
        )
        self.helpFile = os.path.join(tpl_dir, 'chelp', hfile)



# -*- coding: utf-8 -*-
##
## This file is part of the SixPay Indico EPayment Plugin.
## Copyright (C) 2017 Max Fischer
##
## This is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 3 of the
## License, or (at your option) any later version.
##
## This software is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Indico;if not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import, division
import urlparse
import requests
from xml.dom.minidom import parseString

from MaKaC.epayment import BaseEPayMod, BaseTransaction
from MaKaC.common.timezoneUtils import nowutc


from .webinterface import urlHandlers as localUrlHandlers
from . import MODULE_ID


class TransactionError(BaseException):
    """An error occurred inside the transaction to SixPay"""


class SixPayFormatFieldMap(object):
    """
    Mapping lazily providing format fields

    :type registrant: MaKaC.registration.Registrant
    :type conference: MaKaC.conference.Conference
    """
    def __init__(self, registrant, conference):
        self.registrant = registrant
        self.conference = conference
        self._field_cache = {}
        self._field_maker = {
            'user_id': registrant.getId, 'user_name': lambda: registrant.getFullName(title=False, firstNameFirst=True),
            'user_firstname': registrant.getFirstName, 'user_lastname': registrant.getSurName,
            'event_id': conference.getId, 'event_title': conference.getTitle,
            'eventuser_id': registrant.getIdPay,
        }

    def __getitem__(self, key):
        try:
            item = self._field_cache[key]
        except KeyError:
            item = self._field_cache[key] = self._field_maker[key]()
        return item

    def __setitem__(self, key, value):
        self._field_cache[key] = value

    def __repr__(self):
        return '%s(registrant=%s, conference=%s)' % (self.__class__.__name__, self.registrant, self.conference)


class SixPayMod(BaseEPayMod):
    """
    Payment Module for SIX Payment Service

    :param data: mapping to initialise fields
    :type data: dict or None
    """
    #: default settings used when setting fields via `data`
    default_settings = {
        'title': "Six Payment Services",
        'url': "https://www.saferpay.com/hosting",
        'account_id': "",
        'notification_mail': "",
        'user_description': "%(event_title)s, %(user_name)s",
        'order_identifier': "%(eventuser_id)s",
    }

    def __init__(self, data=None):
        BaseEPayMod.__init__(self)
        #: title of this payment method
        self._title = self.default_settings['title']
        #: URL for the saferpay https interface
        self._url = self.default_settings['url']
        #: saferpay account ID of the conference
        self.account_id = self.default_settings['account_id']
        #: description for transaction presented to registrant
        self.user_description = self.default_settings['user_description']
        #: internal description for transaction for organiser and accounting
        self.order_identifier = self.default_settings['order_identifier']
        #: mail to send confirmations to
        self.notification_mail = self.default_settings['notification_mail']
        if data is not None:
            self.setValues(data)

    def getId(self):
        return MODULE_ID

    def clone(self, newSessions):
        """Return a clone of this instance"""
        self_clone = SixPayMod()
        self_clone.setValues(self.getValues())
        self_clone.setEnabled(self.isEnabled())
        return self_clone

    def setValues(self, data):
        """
        Set all fields from mapping

        :param data: mapping to initialise fields
        :type data: dict or None

        :note: any field missing in `data` will be set to its default value
        """
        for key in self.default_settings:
            value = data.get(key) or self.default_settings[key]
            setattr(self, key, value)

    def getValues(self):
        """
        Get all fields as a mapping

        :return: mapping of all fields
        :rtype: dict

        :note: any field missing will be set to its default value
        """
        return {
            key: getattr(self, key, self.default_settings[key]) for key in self.default_settings
        }

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = value

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, value):
        # ensure a trailing slash at end of path, or merging endpoints does not work
        url = urlparse.urlparse(value)
        url = url._replace(path=url.path.rstrip('/') + '/')
        self._url = url.geturl()

    @staticmethod
    def _perform_request(endpoint, **params):
        """Perform a POST request"""
        url_request = requests.post(endpoint, **params)
        # raise any request errors
        url_request.raise_for_status()
        # raise errors in response
        if url_request.text.startswith('ERROR'):
            raise TransactionError('Failed request to SixPay service: %s' % url_request.text)
        return url_request

    def getFormHTML(self, prix, Currency, conf, registrant, lang="en_GB", secure=False):
        """Generates the action of the button presented to the user for payment"""
        payment_url = self._get_payment_url(prix, Currency, conf, registrant, lang, secure)
        payment_action = """<form action="%s" method="POST" id="%s"/>""" % (payment_url, self.getId())
        return payment_action

    def _get_user_description(self, format_map):
        user_description = self.user_description
        user_description %= format_map
        return user_description

    def _get_order_identifier(self, format_map):
        order_identifier = self.order_identifier
        order_identifier %= format_map
        return ''.join(order_identifier.split())

    def _get_payment_url(self, prix, Currency, conf, registrant, lang, secure):
        """
        Generate Payment URL for User

        This method uses the payment details to generate a signed payment URL
        that is secure against manipulation.
        Transmitting the payment request to Six is separate from prompting the
        user to avoid modification of payment details.

        :type registrant: MaKaC.registration.Registrant
        :type conf: MaKaC.conference.Conference
        """
        endpoint = urlparse.urljoin(self.url, 'CreatePayInit.asp')
        # keys for formatting strings
        format_map = SixPayFormatFieldMap(registrant=registrant, conference=conf)
        # description of transaction presented to user
        user_description = self._get_user_description(format_map)
        # internal description of transaction for organiser and accounting
        order_identifier = self._get_order_identifier(format_map)
        # parameters for callbacks so that indico can identify the transaction subject
        callback_params = {'target': conf, 'registrantId': registrant.getId()}
        parameters = {
            'ACCOUNTID': str(self.account_id),
            # indico handles price as largest currency, but six expects smallest
            # e.g. EUR: indico uses 100.2 Euro, but six expects 10020 Cent
            'AMOUNT': '%d' % (prix*100),
            'CURRENCY': Currency,
            'DESCRIPTION': user_description[:50],
            'ORDERID': order_identifier[:80],
            'SHOWLANGUAGES': 'yes',
            # callbacks for the service to redirect users back to indico
            'SUCCESSLINK': localUrlHandlers.UHPayTransactionSuccess.getURL(**callback_params),
            'FAILLINK': localUrlHandlers.UHPayTransactionFaillink.getURL(**callback_params),
            'BACKLINK': localUrlHandlers.UHPayTransactionBacklink.getURL(**callback_params),
            # callback for the service to confirm transaction
            'NOTIFYURL': localUrlHandlers.UHPayTransactionNotifyUrl.getURL(**callback_params),
        }
        if self.notification_mail:
            parameters['NOTIFYADDRESS'] = self.notification_mail
        url_request = self._perform_request(endpoint, data=parameters)
        return str(url_request.text)

    def verify_transaction(self, data, signature, registrant):
        """Ensure the transaction is correct in Indico and SixPay"""
        # DATA: '<IDP
        #           MSGTYPE="PayConfirm" TOKEN="(unused)" VTVERIFY="(obsolete)" KEYID="1-0"
        #           ID="9SUO5zbOGY6OUA45b222bhvrpW2A"
        #           ACCOUNTID="401860-17795278"
        #           PROVIDERID="90"
        #           PROVIDERNAME="Saferpay Test Card"
        #           PAYMENTMETHOD="6"
        #           ORDERID="c281r4"
        #           AMOUNT="100"
        #           CURRENCY="EUR"
        #           IP="141.3.200.120"
        #           IPCOUNTRY="DE"
        #           CCCOUNTRY="US"
        #           MPI_LIABILITYSHIFT="yes"
        #           MPI_TX_CAVV="jAABBIIFmAAAAAAAAAAAAAAAAAA="
        #           MPI_XID="VVE3DQlhXR8PBD5JPzYGWW5FNgI="
        #           ECI="1"
        #           CAVV="jAABBIIFmAAAAAAAAAAAAAAAAAA="
        #           XID="VVE3DQlhXR8PBD5JPzYGWW5FNgI=" />'
        mdom = parseString(data)
        attributes = mdom.documentElement.attributes
        idp_data = {
            attributes.item(idx).name: attributes.item(idx).value
            for idx in range(attributes.length)
        }
        if self._verify_confirmation(data, signature, idp_data=idp_data):
            conference = registrant.getConference()
            format_map = SixPayFormatFieldMap(registrant=registrant, conference=conference)
            transaction = TransactionSixPay(
                user_id=registrant.getId(), event_id=conference.getId(),
                signature=signature, amount=(int(idp_data['AMOUNT']) / 100.0), currency=idp_data['CURRENCY'],
                six_id=idp_data['ID'], order_id=idp_data['ORDERID'], subject=self._get_user_description(format_map)
            )
            # verification may be triggered multiple times
            if registrant.getPayed() and registrant.getTransactionInfo() == transaction:
                return True
            self._complete_transaction(idp_data=idp_data)
            registrant.setTransactionInfo(transaction)
            registrant.setPayed(True)
            registration_form = registrant.getConference().getRegistrationForm()
            registration_form.getNotification().sendEmailNewRegistrantConfirmPay(
                registration_form, registrant
            )
        return False

    def _verify_confirmation(self, data, signature, idp_data):
        """ask for confirmation from SixPay for the signature of the transaction"""
        endpoint = urlparse.urljoin(self.url, 'VerifyPayConfirm.asp')
        url_request = self._perform_request(endpoint, data={'DATA': data, 'SIGNATURE': signature})
        if url_request.text.startswith('OK:'):
            # text = 'OK:ID=56a77rg243asfhmkq3r&TOKEN=%3e235462FA23C4FE4AF65'
            confirmation = dict(key_value.split('=') for key_value in url_request.text.split(':', 1)[1].split('&'))
            if idp_data['ID'] != confirmation['ID'] or idp_data['ID'] != confirmation['ID']:
                raise TransactionError('Mismatched verification and confirmation data')
            return True
        raise RuntimeError("Expected reply 'OK:ID=...&TOKEN=...', got %r" % url_request.text)

    def _complete_transaction(self, idp_data):
        """inform SixPay that we agree to the transaction"""
        endpoint = urlparse.urljoin(self.url, 'PayCompleteV2.asp')
        data = {'ACCOUNTID': idp_data['ACCOUNTID'], 'ID': idp_data['ID']}
        if 'test.' in self.url:
            data['spPassword'] = '8e7Yn5yk'
        url_request = self._perform_request(endpoint, data=data)
        return url_request.text.startswith('OK')

    def getConfModifEPaymentURL(self, conf):
        """URL for applying settings for the EPayment of a single event/conference"""
        return localUrlHandlers.UHConfModifEPaymentSixPay.getURL(conf)


class TransactionSixPay(BaseTransaction):
    """Completed Transaction for SIX Payment Service"""
    def __init__(self, user_id, event_id, signature, amount, currency, six_id, order_id, date=None, subject=None):
        BaseTransaction.__init__(self)
        self.user_id = user_id
        self.event_id = event_id
        self.signature = signature
        self.amount = amount
        self.currency = currency
        self.six_id = six_id
        self.order_id = order_id
        self.date = date or nowutc()
        self.subject = subject or ('Event %s, Registrant %s' % (self.event_id, self.user_id))

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.six_id == other.six_id and self.order_id == other.order_id and self.signature == other.signature
        else:
            return NotImplemented

    def getId(self):
        try:
            return self._id
        except AttributeError:
            self._id = "sixpay"
            return self._id

    def _form_data(self):
        return (
                ('Service', 'SixPay'),
                ('Date', self.date),
                ('Amount', '%.2f %s' % (self.amount, self.currency)),
                ('Description', self.subject),
                ('Identifier', self.order_id),
                ('Transaction ID', self.six_id),
            )

    def getTransactionHTML(self):
        """HTML for displaying transaction in Indico"""
        entry = """\
                          <tr>
                            <td align="right"><b>%s</b></td>
                            <td align="left">%s</td>
                          </tr>"""
        return """\
                        <table>
%s
                        </table>""" % '\n'.join(
            entry % (name, value)
            for name, value in
            self._form_data()
        )

    def getTransactionTxt(self):
        """Text for displaying transaction in mails"""
        entry = """%-16s %s"""
        return '\n'.join(
            entry % (name, value)
            for name, value in
            self._form_data()
        )


def getPayMod():
    return SixPayMod()


def getPayModClass():
    return SixPayMod



# -*- coding: utf-8 -*-
##
## This file is part of the SixPay Indico EPayment Plugin.
## Copyright (C) 2017 Max Fischer
##
## This is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 3 of the
## License, or (at your option) any later version.
##
## This software is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Indico;if not, see <http://www.gnu.org/licenses/>.

from MaKaC.webinterface.rh.ePaymentModif import RHEPaymentModifBase, RHRegistrationFormDisplayBase


from ..pages import ePayments
from .. import urlHandlers as localUrlHandlers
from ... import epayment as ePayment
from ... import MODULE_ID


# Editor's Note  - MF@20170309
# These classes implement the actual functionality. The URLHandlers link incoming
# request to RequestHandlers *by sharing the _requestTag field*. RequestHandlers
# actually digest data and modify state. They use the pages.ePayments to *display*
# results and data.


class RHEPaymentmodifSixPay(RHEPaymentModifBase):
    _requestTag = "modifSixPay"

    def _process(self):
        p = ePayments.WPConfModifEPaymentSixPay(self, self._conf)
        return p.display()


class RHEPaymentmodifSixPayDataModif(RHEPaymentModifBase):
    _requestTag = "modifSixPayData"

    def _process(self):
        p = ePayments.WPConfModifEPaymentSixPayDataModif(self, self._conf)
        return p.display()


class RHEPaymentmodifSixPayPerformDataModif(RHEPaymentModifBase):
    _requestTag = "modifSixPayPerformDataModif"

    def _checkParams(self, params):
        RHEPaymentModifBase._checkParams(self, params)
        self._cancel = params.has_key("cancel")

    def _process(self):
        if not self._cancel:
            conference_six_epayment = self._conf.getModPay().getPayModByTag(MODULE_ID)
            data = self._getRequestParams()
            conference_six_epayment.setValues(data)
        self._redirect(localUrlHandlers.UHConfModifEPaymentSixPay.getURL(self._conf))


# Handlers for the Six Pay service to call back after/during a transaction
class RHTransactionUserCallback(RHRegistrationFormDisplayBase):
    """Request Handler for Callbacks the User is directed to"""
    #: display page after handling transaction
    display_page = None
    #: overwrite message on the displayed page
    message = None
    #: overwrite message detail on the displayed page
    message_detail = None

    def _checkParams(self, params):
        RHRegistrationFormDisplayBase._checkParams(self, params)
        self._registrant = None
        regId = params.get("registrantId", "")
        if regId is not None:
            self._registrant = self._conf.getRegistrantById(regId)

    def _processIfActive(self):
        if self._registrant is not None:
            assert self.display_page is not None, "Callbacks must set display pages for users"
            display_page = self.display_page(self, self._conf, self._registrant)
            if self.message is not None:
                display_page.message = self.message
            if self.message_detail is not None:
                display_page.message_detail = self.message_detail
            return display_page.display()


class RHTransactionSuccesslink(RHTransactionUserCallback):
    """redirect when the user completed the transaction"""
    _requestTag = "successlink"
    display_page = ePayments.WPTransactionSuccesslink

    def _checkParams(self, params):
        # indico tries to evaluate the DATA xml as an HTML locator if we do not remove it
        data = params.pop('DATA')
        RHTransactionUserCallback._checkParams(self, params)
        conference_six_epayment = self._conf.getModPay().getPayModByTag(MODULE_ID)
        try:
            conference_six_epayment.verify_transaction(
                data=data,
                signature=params['SIGNATURE'],
                registrant=self._registrant,
            )
        except ePayment.TransactionError as transaction_error:
            self.message = "Your payment has been processed by SixPay, but a validation error occurred"
            self.message_detail = 'Please contact the event organizers to confirm your payment\n%s' % transaction_error


class RHTransactionFaillink(RHTransactionUserCallback):
    """redirect when the user could not be authorised"""
    _requestTag = "faillink"
    display_page = ePayments.WPTransactionFaillink


class RHTransactionBacklink(RHTransactionUserCallback):
    """redirect when the user aborts the transaction"""
    _requestTag = "backlink"
    display_page = ePayments.WPTransactionBacklink


# Handler called separate of user redirects after successfull transaction
# This sends the verification independent of any calls the user may or may not perform.
class RHTransactionNotifyUrl(RHTransactionSuccesslink):
    """endpoint for SixPay to confirm transaction WITHOUT user intervention"""
    _requestTag = "notifyurl"

    def _checkProtection(self):
        # Just bypass everything else, as we want the payment service
        # to acknowledge the payment
        pass

    def _process(self):
        # Everything has been handled by confirming the request, do not expose any more options
        pass



from setuptools import setup, find_packages

setup(
    name='indico_sixpay',
    version='1.2.2',
    description='Indico EPayment Sub-Plugin to use SixPay services',
    url='https://github.com/maxfischer2781/indico_sixpay',
    author='Max Fischer',
    author_email='maxfischer2781@gmail.com',
    entry_points={'indico.ext': ['EPayment.sixPay = indico_sixpay', ], },
    packages=find_packages(),
    package_data={'indico_sixpay': ['tpls/*.tpl']},
    install_requires=['requests', 'indico>=1.2'],
    license='GPLv3+',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Plugins',
        'Intended Audience :: Science/Research',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Communications :: Conferencing',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
    ],
    zip_safe=False,
    keywords='indico epayment six sixpay plugin',
)



# -*- coding: utf-8 -*-
##
## This file is part of the SixPay Indico EPayment Plugin.
## Copyright (C) 2017 Max Fischer
##
## This is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 3 of the
## License, or (at your option) any later version.
##
## This software is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Indico;if not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import, division
from MaKaC.webinterface.urlHandlers import URLHandler as MainURLHandler

from .. import MODULE_ID


# Editor's Note  - MF@20170309
# These classes simply define URLs *on the indico server* which are open handles
# for external requests. The requestTag is used to search corresponding pages/actions
# in other submodules.


class EPURLHandler(MainURLHandler):
    _requestTag = ''

    @classmethod
    def getURL(cls, target=None, _ignore_static=False, **params):
        return super(EPURLHandler, cls).getURL(
            target, EPaymentName=MODULE_ID, requestTag=cls._requestTag, _ignore_static=False, **params
        )


# URLs for Indico to modify the plugin
class UHConfModifEPayment(EPURLHandler):
    _endpoint = 'event_mgmt.confModifEpayment-modifModule'


class UHConfModifEPaymentSixPay(UHConfModifEPayment):
    _requestTag = "modifSixPay"


class UHConfModifEPaymentSixPayDataModif(UHConfModifEPayment):
    _requestTag = "modifSixPayData"


class UHConfModifEPaymentSixPayPerformDataModif(UHConfModifEPayment):
    _requestTag = "modifSixPayPerformDataModif"


class UHPay(EPURLHandler):
    _endpoint = 'misc.payment'


# URLs given to the Six Pay service to call back after/during a transaction
class UHPayTransactionSuccess(UHPay):
    # redirect when the user completed the transaction
    _requestTag = "successlink"


class UHPayTransactionFaillink(UHPay):
    # redirect when the user could not be authorised
    _requestTag = "faillink"


class UHPayTransactionBacklink(UHPay):
    # redirect when the user aborts the transaction
    _requestTag = "backlink"


class UHPayTransactionNotifyUrl(UHPay):
    # endpoint for SixPay to confirm transaction WITHOUT user intervention
    _requestTag = "notifyurl"
